@@ -1,21 +1,21 @@
 /*     -*-C++-*- -*-coding: utf-8-unix;-*-
-       Classified Ads is Copyright (c) Antti Järvinen 2013.
+  Classified Ads is Copyright (c) Antti Järvinen 2013-2016.
 
-       This file is part of Classified Ads.
+  This file is part of Classified Ads.
 
-    Classified Ads is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+  Classified Ads is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
-    Classified Ads is distributed in the hope that it will be useful,
-       but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+  Classified Ads is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Classified Ads; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+  You should have received a copy of the GNU Lesser General Public
+  License along with Classified Ads; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include "model.h"
@@ -47,6 +47,7 @@
 #include "privmsgmodel.h"
 #include "profilecommentmodel.h"
 #include "searchmodel.h"
+#include "cadbrecordmodel.h"
 #include "trusttreemodel.h"
 #include <QMutex>
 #include <QTimerEvent>
@@ -55,6 +56,7 @@
 #include <windows.h>
 #include <Wincrypt.h>
 #endif
+#include "tclmodel.h"
 
 Model::Model(MController *aController) : iController(aController),
     iMutex(NULL) ,
@@ -70,8 +72,10 @@ Model::Model(MController *aController) : iController(aController),
     iPrivMsgModel(NULL),
     iProfileCommentModel(NULL),
     iSearchModel(NULL),
+    iCaDbRecordModel(NULL),
     iTrustTreeModel(NULL),
-    iTimeOfLastNetworkAddrCheck(QDateTime::currentDateTimeUtc().toTime_t()) {
+    iTimeOfLastNetworkAddrCheck(QDateTime::currentDateTimeUtc().toTime_t()),
+    iTclModel(NULL) {
     LOG_STR("Model::Model()") ;
     connect(this,
             SIGNAL(  error(MController::CAErrorSituation,
@@ -99,9 +103,12 @@ Model::Model(MController *aController) : iController(aController),
     iPrivMsgModel = new PrivMessageModel(aController, *this) ;
     iSearchModel = new SearchModel(*this,*iController) ;
     iSearchModel->setObjectName("CA SearchModel") ;
+    iCaDbRecordModel = new CaDbRecordModel(aController,
+                                           *this);
     iTrustTreeModel = new TrustTreeModel(aController, *this ) ;
     iProfileCommentModel = new ProfileCommentModel(aController, *this) ;
     iNetReqExecutor->setInterval(1000); // start is called by controller
+    iTclModel = new TclModel(aController, *this) ;
 
     connect(iBinaryFileModel,
             SIGNAL( contentReceived(const Hash& ,
@@ -208,6 +215,17 @@ Model::Model(MController *aController) : iController(aController),
                                          const ProtocolItemType)),
             Qt::QueuedConnection) ;
 
+    connect(iCaDbRecordModel,
+            SIGNAL( contentReceived(const Hash& ,
+                                    const Hash& ,
+                                    const ProtocolItemType ) ),
+            iController,
+            SLOT(notifyOfContentReceived(const Hash& ,
+                                         const Hash& ,
+                                         const ProtocolItemType)),
+            Qt::QueuedConnection) ;
+
+
     iTimerId = startTimer(60000*10);  // 10-minute timer
 }
 
@@ -230,8 +248,10 @@ Model::~Model() {
     delete iProfileModel ;
     delete iNetReqQueue ;
     delete iBinaryFileModel ;
+    delete iCaDbRecordModel ; 
     delete iSearchModel ;
     delete iTrustTreeModel ;
+    delete iTclModel ;
     delete iCaModel ;
     delete iContentEncryptionModel ;
     iDb.close();
@@ -316,7 +336,16 @@ bool Model::openDB() {
             QLOG_STR("Cleaned away faulty comments " + QString::number(query.numRowsAffected () )) ;
         }
     }
-    createTablesV2() ; 
+
+    if ( isDbOpen ) {
+        createTablesV2() ;
+        createTablesV3() ;
+        createTablesV4() ;
+        QSqlQuery versionQuery("select sqlite_version()") ;
+        if ( versionQuery.exec() && versionQuery.next() ) {
+            QLOG_STR("Sqlite version " + versionQuery.value(0).toString()) ; 
+        }
+    }
     return isDbOpen ;
 }
 
@@ -598,7 +627,6 @@ bool Model::createTablesV2() {
 
     QSqlQuery query;
     bool ret ;
-    // don't pick up the same item again if less than 10 minutes passed
     ret = query.prepare ("select datastore_version from settings" ) ;
     ret = query.exec() ;
 
@@ -611,24 +639,163 @@ bool Model::createTablesV2() {
             if ( datastoreVersion == 1 ) {
                 // upgrade datastore to version 2
                 QSqlQuery q;
-                ret = q.exec("alter table settings add column ringtone int not null default 0") ; 
+                ret = q.exec("alter table settings add column ringtone int not null default 0") ;
                 if ( ret ) {
                     QSqlQuery q3;
-                    ret = q3.exec("alter table settings add column call_acceptance int not null default 0") ; 
+                    ret = q3.exec("alter table settings add column call_acceptance int not null default 0") ;
+                    if ( !ret ) {
+                        QLOG_STR("alter table settings: " + q3.lastError().text());
+                        emit error(MController::DbTransactionError, q3.lastError().text()) ;
+                    }
                 }
                 if ( ret ) {
                     QSqlQuery q2;
-                    q2.exec("update settings set datastore_version = 2") ; 
+                    q2.exec("update settings set datastore_version = 2") ;
+                } 
+            }
+        }
+    }
+    return ret ;
+}
+
+// method check datastore version. if is 2, upgrades to version 3.
+bool Model::createTablesV3() {
+
+    QSqlQuery query;
+    bool ret ;
+    ret = query.prepare ("select datastore_version from settings" ) ;
+    ret = query.exec() ;
+
+    if ( !ret ) {
+        QLOG_STR("select datastore_version from settings: " + query.lastError().text());
+        emit error(MController::DbTransactionError, query.lastError().text()) ;
+    } else {
+        if ( query.next() && !query.isNull(0) ) {
+            int datastoreVersion ( query.value(0).toInt() ) ;
+            if ( datastoreVersion == 2 ) {
+                // upgrade datastore to version 3
+                QSqlQuery q;
+                ret = q.exec("create table tclprogs "
+                             "(hash1 unsigned int not null,"
+                             "hash2 unsigned int not null,"
+                             "hash3 unsigned int not null,"
+                             "hash4 unsigned int not null,"
+                             "hash5 unsigned int not null,"
+                             "prog_name varchar(256) not null,"
+                             "prog_text blob not null,"
+                             "prog_settings blob,"
+                             "time_modified unsigned int not null)") ;
+                if ( ret ) {
+                    QSqlQuery q2;
+                    q2.exec("update settings set datastore_version = 3") ;
                 } else {
-                    QLOG_STR("alter table settings: " + query.lastError().text());
-                    emit error(MController::DbTransactionError, query.lastError().text()) ;
+                    QLOG_STR("alter table settings: " + q.lastError().text());
+                    emit error(MController::DbTransactionError, q.lastError().text()) ;
                 }
             }
         }
     }
-    return ret ; 
+    return ret ;
 }
 
+bool Model::createTablesV4() {
+    QLOG_STR("createTablesV4 in") ; 
+    QSqlQuery query;
+    bool ret ;
+    ret = query.prepare ("select datastore_version from settings" ) ;
+    ret = query.exec() ;
+
+    if ( !ret ) {
+        QLOG_STR("select datastore_version from settings: " + query.lastError().text());
+        emit error(MController::DbTransactionError, query.lastError().text()) ;
+    } else {
+        if ( query.next() && !query.isNull(0) ) {
+            int datastoreVersion ( query.value(0).toInt() ) ;
+            if ( datastoreVersion == 3 ) {
+                // upgrade datastore to version 4
+                QSqlQuery q;
+                ret = q.exec("create table dbrecord ( hash1 unsigned int not null,"
+                             "hash2 unsigned int not null,"
+                             "hash3 unsigned int not null,"
+                             "hash4 unsigned int not null,"
+                             "hash5 unsigned int not null,"
+                             "collection_hash1 unsigned int not null,"
+                             "collection_hash2 unsigned int not null,"
+                             "collection_hash3 unsigned int not null,"
+                             "collection_hash4 unsigned int not null,"
+                             "collection_hash5 unsigned int not null,"
+                             "sender_hash1 unsigned int not null,"
+                             "sender_hash2 unsigned int not null,"
+                             "sender_hash3 unsigned int not null,"
+                             "sender_hash4 unsigned int not null,"
+                             "sender_hash5 unsigned int not null,"
+                             "time_last_reference unsigned int not null,"
+                             "signature varchar(1024) not null,"
+                             "time_of_publish unsigned int not null,"
+                             "data blob," 
+                             "recvd_from  unsigned int not null,"
+                             "searchstring varchar(4096)," 
+                             "searchnumber bigint,"
+                             "isencrypted tinyint not null," 
+                             "issignatureverified tinyint not null);") ;// will store boolean value
+                if (!ret ) {
+                    QLOG_STR("create table dbrecord: " + q.lastError().text());
+                    emit error(MController::DbTransactionError, q.lastError().text()) ;
+                } 
+                if ( ret && SearchModel::queryIfFTSSupported() ) {
+                    QSqlQuery ftsQ ;
+                    ret = ftsQ.exec("create virtual table dbrecord_search using fts3(searchstring)") ;
+                    if (!ret) {
+                        QLOG_STR("FTS3 table dbrecord_search creation: " + ftsQ.lastError().text()) ;
+                        return false ;
+                    } else {
+                        QSqlQuery ftsQ2 ;
+                        ret = ftsQ2.exec("CREATE TRIGGER dbrecord_search_del BEFORE DELETE ON dbrecord BEGIN"
+                                         "  DELETE FROM dbrecord_search WHERE docid=old.hash1;"
+                                         "END;") ;
+                        if (!ret) {
+                            QLOG_STR("CREATE TRIGGER dbrecord_search_del: " + ftsQ2.lastError().text());
+                            emit error(MController::DbTransactionError, ftsQ2.lastError().text()) ;
+                        }
+                        
+                    }
+                } else {
+                    QLOG_STR("FTS was not supported") ;
+                }
+                if ( ret ) {
+                    QSqlQuery q3;
+                    ret = q3.exec("alter table settings add column dbrecord_maxrows int not null default 1000") ;
+                    if (!ret) {
+                        QLOG_STR("add dbrecord_maxrows: " + q3.lastError().text()) ;
+                        emit error(MController::DbTransactionError, q3.lastError().text()) ;
+                        return false ;
+                    }
+                    QSqlQuery q4;
+                    ret = q4.exec("update settings set dbrecord_maxrows = 100000") ;
+                    if (!ret) {
+                        QLOG_STR("update dbrecord_maxrows: " + q4.lastError().text()) ;
+                        emit error(MController::DbTransactionError, q4.lastError().text()) ;
+                        return false ;
+                    }
+                    QSqlQuery q5;
+                    ret = q5.exec("alter table tclprogs add column prog_data blob") ;
+                    if (!ret) {
+                        QLOG_STR("alter table tclprogs: " + q5.lastError().text()) ;
+                        emit error(MController::DbTransactionError, q5.lastError().text()) ;
+                        return false ;
+                    }
+                }
+                if ( ret ) {
+                    QSqlQuery q2;
+                    q2.exec("update settings set datastore_version = 4") ;
+                } 
+            }
+        }
+    }
+    return ret ;
+}
+
+
 void Model::initPseudoRandom() {
     SSL_load_error_strings();
     SSL_library_init();
@@ -807,8 +974,8 @@ void Model::connectionStateChanged(Connection::ConnectionState aState,
         foreach ( const Connection *c, *iConnections ) {
             if ( c ) {
                 if ( c != aConnection &&
-                     c->connectionState() == Connection::Open &&
-                     aConnection->getPeerHash() == c->getPeerHash() ) {
+                        c->connectionState() == Connection::Open &&
+                        aConnection->getPeerHash() == c->getPeerHash() ) {
                     LOG_STR("Double connection to same node detected") ;
                     aConnection->iNeedsToRun = false ;
                     break ;
@@ -848,8 +1015,8 @@ void Model::addItemToSend(const Hash& aDestinationNode,
         c = iConnections->value(i) ;
         nodeOfConnection = c->node() ;
         if (c->connectionState() == Connection::Open &&
-            nodeOfConnection && 
-            ( nodeOfConnection->nodeFingerPrint() == aDestinationNode ) ) {
+                nodeOfConnection &&
+                ( nodeOfConnection->nodeFingerPrint() == aDestinationNode ) ) {
             // yes.
             c->iNextProtocolItemToSend.append(aBytesToSend) ;
             return ; // work done, get out
@@ -888,9 +1055,17 @@ ProfileCommentModel& Model::profileCommentModel() const {
 SearchModel* Model::searchModel() const {
     return iSearchModel ;
 }
+CaDbRecordModel* Model::caDbRecordModel() const {
+    return iCaDbRecordModel ;
+}
 TrustTreeModel* Model::trustTreeModel() const {
     return iTrustTreeModel ;
 }
+
+TclModel& Model::tclModel() const {
+    return *iTclModel ;
+}
+
 NetworkRequestExecutor* Model::getNetReqExecutor() {
     return iNetReqExecutor ;
 }
@@ -1349,7 +1524,14 @@ void Model::timerEvent(QTimerEvent*
     iProfileModel->truncateDataTableToMaxRows() ;
     unlock() ;
     QThread::yieldCurrentThread ();
-
+    lock() ;
+    iCaDbRecordModel->truncateDataTableToMaxRows() ;
+    unlock() ;
+    QThread::yieldCurrentThread ();
+    lock() ;
+    QSqlQuery vacuumQuery ; 
+    vacuumQuery.exec("vacuum") ;
+    unlock() ;
     // then check out for local network addresses:
     if ( (iTimeOfLastNetworkAddrCheck + (120*60)) < // every 120 minutes
             QDateTime::currentDateTimeUtc().toTime_t()) {
@@ -1363,10 +1545,10 @@ void Model::timerEvent(QTimerEvent*
 
 
 Model::RingtoneSetting Model::getRingtoneSetting() {
-    RingtoneSetting retval ( BowRingTone ) ; 
+    RingtoneSetting retval ( BowRingTone ) ;
     QSqlQuery query;
     bool ret ;
-    
+
     ret = query.prepare ("select ringtone from settings" ) ;
     ret = query.exec() ;
 
@@ -1378,7 +1560,7 @@ Model::RingtoneSetting Model::getRingtoneSetting() {
             retval = (Model::RingtoneSetting)(query.value(0).toInt());
         }
     }
-    return retval ; 
+    return retval ;
 }
 
 void Model::setRingtoneSetting(Model::RingtoneSetting aRingTone) {
@@ -1393,10 +1575,10 @@ void Model::setRingtoneSetting(Model::RingtoneSetting aRingTone) {
 
 
 Model::CallAcceptanceSetting Model::getCallAcceptanceSetting() {
-    CallAcceptanceSetting retval ( AcceptAllCalls ) ; 
+    CallAcceptanceSetting retval ( AcceptAllCalls ) ;
     QSqlQuery query;
     bool ret ;
-    
+
     ret = query.prepare ("select call_acceptance from settings" ) ;
     ret = query.exec() ;
 
@@ -1408,7 +1590,7 @@ Model::CallAcceptanceSetting Model::getCallAcceptanceSetting() {
             retval = (Model::CallAcceptanceSetting)(query.value(0).toInt());
         }
     }
-    return retval ; 
+    return retval ;
 }
 
 void Model::setCallAcceptanceSetting(Model::CallAcceptanceSetting aAcceptance) {
@@ -1420,3 +1602,4 @@ void Model::setCallAcceptanceSetting(Model::CallAcceptanceSetting aAcceptance) {
     }
 }
 
+
