@@ -1,21 +1,21 @@
 /*     -*-C++-*- -*-coding: utf-8-unix;-*-
-   Classified Ads is Copyright (c) Antti Järvinen 2013.
+  Classified Ads is Copyright (c) Antti Järvinen 2013-2017.
 
-   This file is part of Classified Ads.
+  This file is part of Classified Ads.
 
-    Classified Ads is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+  Classified Ads is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
-    Classified Ads is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+  Classified Ads is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Classified Ads; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+  You should have received a copy of the GNU Lesser General Public
+  License along with Classified Ads; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 #include "publishingengine.h"
 #include "../log.h"
@@ -84,13 +84,10 @@ void PublishingEngine::run() {
                     QList<Node *>* nodesToTry = NULL ;
                     switch ( iWorkItem.iObjectType ) {
                     case ClassifiedAd2NdAddr:
-                        nodesToTry    =
-                            iModel.nodeModel().getNodesAfterHash(iWorkItem.i2NdAddr,
-                                    20, // 20 nodes
-                                    300 ) ;// at most 5 hours old
-                        break ;
                     case UserProfileComment:
-                        // user profile commments get published to profile address,
+                    case DbRecord:
+                        // user profile commments, ads with 2nd addr and 
+                        // db records get published to seconddary address,
                         // not content address
                         nodesToTry    =
                             iModel.nodeModel().getNodesAfterHash(iWorkItem.i2NdAddr,
@@ -98,6 +95,8 @@ void PublishingEngine::run() {
                                     300 ) ;// at most 5 hours old
                         break ;
                     default:
+                        // all other content types get published to
+                        // primary address, or addr of the object itself
                         nodesToTry    =
                             iModel.nodeModel().getNodesAfterHash(iWorkItem.iObjectHash,
                                     20, // 20 nodes
@@ -316,6 +315,17 @@ void PublishingEngine::publishToNode(const Hash& aNode) {
                     LOG_STR2("Already published to to node low-order bits %u", iWorkItem.iAlreadyPushedHosts[i]) ;
                 }
                 break ;
+            case DbRecord:
+                sendReq.iRequestType = DbRecordPublish ;
+                iModel.addNetworkRequest(sendReq) ;
+                iWorkItem = iModel.addNodeToPublishedItem(iWorkItem.iObjectHash, aNode) ;
+                LOG_STR2("Published db record to node %s", qPrintable(aNode.toString())) ;
+                LOG_STR2("Published db record to node low-order bits %u", aNode.iHash160bits[4]) ;
+                LOG_STR2("Now work-item type is %d",(int) iWorkItem.iObjectType) ;
+                for ( int i = 0 ; i <iWorkItem.iAlreadyPushedHosts.size() ; i++ ) {
+                    LOG_STR2("Already published to to node low-order bits %u", iWorkItem.iAlreadyPushedHosts[i]) ;
+                }
+                break ;
             default:
                 LOG_STR2("TODO: unimplemented publish of type %d", iWorkItem.iObjectType) ;
                 break ;
