@@ -1,21 +1,21 @@
-/*     -*-C++-*- -*-coding: utf-8-unix;-*-
-       Classified Ads is Copyright (c) Antti Järvinen 2013.
+/*   -*-C++-*- -*-coding: utf-8-unix;-*-
+  Classified Ads is Copyright (c) Antti Järvinen 2013-2016.
 
-       This file is part of Classified Ads.
+  This file is part of Classified Ads.
 
-    Classified Ads is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+  Classified Ads is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
-    Classified Ads is distributed in the hope that it will be useful,
-       but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+  Classified Ads is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Classified Ads; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+  You should have received a copy of the GNU Lesser General Public
+  License along with Classified Ads; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include <QtGui>
@@ -27,6 +27,7 @@
 #include "../datamodel/camodel.h"
 #include "../datamodel/privmsgmodel.h"
 #include "../datamodel/profilecommentmodel.h"
+#include "../datamodel/cadbrecordmodel.h"
 #include "settings.h"
 
 SettingsDialog::SettingsDialog(QWidget *aParent,
@@ -46,6 +47,7 @@ SettingsDialog::SettingsDialog(QWidget *aParent,
     ui.nrBinaryFilesEdit->setValue ( iController.model().binaryFileModel().getMaxRowsToKeep() );
     ui.nrAdsEdit->setValue ( iController.model().classifiedAdsModel().getMaxRowsToKeep() );
     ui.nrCommenstEdit->setValue ( iController.model().profileCommentModel().getMaxRowsToKeep() );
+    ui.nrDbRecordsEdit->setValue ( iController.model().caDbRecordModel()->getMaxRowsToKeep() );
     ui.dnsNameEdit->setText( iController.getNode().DNSAddr()) ;
     ui.ringtoneComboBox->setCurrentIndex(
         (int)(iController.model().getRingtoneSetting())) ;
@@ -70,6 +72,7 @@ void SettingsDialog::okButtonClicked() {
     iController.model().binaryFileModel().setMaxRowsToKeep(ui.nrBinaryFilesEdit->value (  ));
     iController.model().classifiedAdsModel().setMaxRowsToKeep(ui.nrAdsEdit->value (  ));
     iController.model().profileCommentModel().setMaxRowsToKeep(ui.nrCommenstEdit->value (  )) ;
+    iController.model().caDbRecordModel()->setMaxRowsToKeep(ui.nrDbRecordsEdit->value (  )) ;
     iController.getNode().setDNSAddr(ui.dnsNameEdit->text( ) ) ;
     iController.model().setRingtoneSetting(
         (Model::RingtoneSetting)(ui.ringtoneComboBox->currentIndex())) ;
